@@ -7,7 +7,7 @@ from datetime import datetime
 from typing import Dict, Union, Optional
 
 import rdflib
-from pydantic import HttpUrl, FileUrl, BaseModel, ConfigDict
+from pydantic import HttpUrl, FileUrl, BaseModel, ConfigDict, field_validator, Field
 from rdflib.plugins.shared.jsonld.context import Context
 
 from .decorator import urirefs, namespaces, URIRefManager, NamespaceManager, _is_http_url
@@ -19,7 +19,7 @@ logger = logging.getLogger('ontolutils')
 EXTRA = 'allow'  # or 'forbid' or 'ignore'
 
 
-class ThingModel(BaseModel, abc.ABC):
+class ThingModel(BaseModel, abc.ABC, validate_assignment=True):
     """Abstract base model class to be used by model classes used within ontolutils"""
 
     model_config = ConfigDict(extra=EXTRA, populate_by_name=True)
@@ -44,7 +44,8 @@ def resolve_iri(key_or_iri: str, context: Context) -> Optional[str]:
             return 'http://www.w3.org/2000/01/rdf-schema#label'
     return
 
-
+def build_blank_n3():
+    return rdflib.BNode().n3()
 @namespaces(owl='http://www.w3.org/2002/07/owl#',
             rdfs='http://www.w3.org/2000/01/rdf-schema#')
 @urirefs(Thing='owl:Thing', label='rdfs:label')
@@ -94,9 +95,16 @@ class Thing(ThingModel):
     >>> #     For further information visit https://errors.pydantic.dev/2.4/v/string_type
 
     """
-    id: Union[str, HttpUrl, FileUrl, BlankNodeType, None] = None  # @id
+    id: Optional[Union[str, HttpUrl, FileUrl, BlankNodeType, None]] = Field(default_factory=build_blank_n3)  # @id
     label: str = None  # rdfs:label
 
+    @field_validator('id')
+    @classmethod
+    def _id(cls, id: Optional[Union[str, HttpUrl, FileUrl, BlankNodeType]]) -> str:
+        if id is None:
+            return rdflib.BNode().n3()
+        return str(id)
+
     def __lt__(self, other: ThingModel) -> bool:
         """Less than comparison. Useful to sort Thing objects.
         Comparison can only be done with other Thing objects and if an ID is given.
@@ -107,16 +115,6 @@ class Thing(ThingModel):
             return False
         return self.id <= other.id
 
-    def pop_blank_node_id(self) -> Union[None, str]:
-        """Remove the ID if it is blank node"""
-        if self.id is None:
-            return
-        if self.id.startswith('http'):
-            return
-        _id = self.id
-        self.id = None
-        return _id
-
     def get_jsonld_dict(self,
                         context: Optional[Union[Dict, str]] = None,
                         exclude_none: bool = True,
@@ -349,7 +347,10 @@ class Thing(ThingModel):
 
     def __repr__(self, limit: Optional[int] = None):
         _fields = {k: getattr(self, k) for k in self.model_fields if getattr(self, k) is not None}
-        repr_extra = ", ".join([f"{k}={v}" for k, v in {**_fields, **self.model_extra}.items()])
+        if self.model_extra:
+            repr_extra = ", ".join([f"{k}={v}" for k, v in {**_fields, **self.model_extra}.items()])
+        else:
+            repr_extra = ", ".join([f"{k}={v}" for k, v in {**_fields}.items()])
         if limit is None or len(repr_extra) < limit:
             return f"{self.__class__.__name__}({repr_extra})"
         return f"{self.__class__.__name__}({repr_extra[0:limit]}...)"
